'use strict';
require('pick-precompiled').babelPolyfill()

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var crypto = require('crypto');
crypto = 'default' in crypto ? crypto['default'] : crypto;
var rest = require('restler');
rest = 'default' in rest ? rest['default'] : rest;
var js2xml = require('js2xml');

/**
 * Generate access token for shop
 *
 * Examples:
 *
 *      insales.token('12345');
 *
 * Output:
 *
 *      8cfa2282b17de0a598c010f5f0109e7d
 * @param {String} token Required, token receive when install app
 * @param {String} secret Required, app secret
 * @returns {String} access token
 */
function _token(token, secret) {
  return crypto.createHash('md5').update(token + secret).digest('hex');
}

var options = {
  headers: {
    'Content-Type': 'application/xml'
  },
  xml2js: {
    trim: false,
    explicitArray: false,
    ignoreAttrs: true
  },
  timeout: 15000
};

function error(type, url, msg, response) {
  return {
    type: type,
    url: url,
    msg: msg,
    response: response
  };
}

function output(data, response) {
  return {
    data: data,
    response: response
  };
}

function request(r) {
  return new Promise(function (resolve, reject) {
    r.once('timeout', function (ms) {
      reject(error('Timeout', r.url.href, 'timeout: ' + ms + 'ms'));
    }).once('error', function (err, response) {
      reject(error('Error', r.url.href, err, response));
    }).once('abort', function () {
      reject(error('Abort', r.url.href, null, null));
    }).once('fail', function (data, response) {
      reject(error('Fail', r.url.href, data, response));
    }).once('success', function (data, response) {
      resolve(output(data, response));
    });
  });
}

var opt = Object.create(options);

function get(conn) {
  if (conn.query) opt.query = conn.query;

  var g = rest.get('https://' + conn.id + ':' + conn.token + '@' + conn.url + '/admin/' + conn.api + '.xml', opt);
  return new Promise(function (resolve, reject) {
    request(g).then(resolve).catch(reject);
  });
}

/**
 * GET /admin/account.xml
 *
 * Examples:
 *
 *      insales.getAccount({
 *        token: '1234567890',
 *        url: 'shop-test.myinsales.ru'
 *      }).then(response => {
 *        console.info(response.data);
 *      }).catch(err => {
 *        console.error(err);
 *      });
 * Output:
 *
 *      { account:
 *         { blocked: 'false',
 *           city: 'Москва',
 *           'contact-phone': '',
 *           country: 'RU',
 *           'enable-cart-discounts': 'true',
 *           'enable-client-discounts': 'false',
 *           'enable-group-discounts': 'true',
 *           'enable-order-discounts': 'false',
 *           'hide-items-out-of-stock': 'false',
 *           id: '12345',
 *           'notification-email': '',
 *           organization: '',
 *           'sms-notification-phone': '',
 *           state: 'г Москва',
 *           subdomain: 'shop-test',
 *           email: 'mail@mail.ru',
 *           icq: '',
 *           phone: '+71234567890',
 *           title: 'Название магазина',
 *           'minimum-items-price': '',
 *           'stock-currency-exchange-rate': '1.0',
 *           'client-cookies-whitelist': '',
 *           'main-host': 'shop-test.myinsales.ru',
 *           'next-order-number': '1040',
 *           'time-zone': 'Moscow',
 *           owner:
 *            { 'created-at': '2011-11-11 11:11:11 +0400',
 *              email: 'mail@mail.ru',
 *              id: '12345',
 *               name: 'Имя' } } }
 * @param {Object} options
 * @param {string} options.token Required, access token for shop
 * @param {string} options.url Required, shop url
 * @param {number|string} id Required, app id
 * @returns {Promise}
 */
function _getAccount(options, id) {
  return new Promise(function (resolve, reject) {
    get({
      id: id,
      token: options.token,
      url: options.url,
      api: 'account'
    }).then(resolve).catch(reject);
  });
}

var opt$1 = Object.create(options);

function post(conn) {
  if (conn.obj !== undefined) {
    var js2xml$$ = new js2xml.Js2Xml(Object.keys(conn.obj)[0], conn.obj[Object.keys(conn.obj)[0]]);
    opt$1.data = js2xml$$.toString();
  }

  var p = rest.post('https://' + conn.id + ':' + conn.token + '@' + conn.url + '/admin/' + conn.api + '.xml', opt$1);
  return new Promise(function (resolve, reject) {
    request(p).then(resolve).catch(reject);
  });
}

function _getProduct(options, id) {
  return new Promise(function (resolve, reject) {
    get({
      id: id,
      token: options.token,
      url: options.url,
      api: 'products/' + options.productid
    }).then(resolve).catch(reject);
  });
}

function _createProduct(options, id) {
  return new Promise(function (resolve, reject) {
    post({
      id: id,
      token: options.token,
      url: options.url,
      api: 'products',
      obj: options.product
    }).then(resolve).catch(reject);
  });
}

function _listProduct(options, id) {
  var query = {};
  query.page = options.page ? options.page : 1;
  query.per_page = options.per_page ? options.per_page : 250;
  if (options.updated_since && options.updated_since !== false) {
    query.updated_since = options.updated_since;
  }
  if (options.deleted === true) {
    query.deleted = true;
  }

  return new Promise(function (resolve, reject) {
    get({
      id: id,
      token: options.token,
      url: options.url,
      api: 'products',
      query: query
    }).then(resolve).catch(reject);
  });
}

function _getCollection(options, id) {
  return new Promise(function (resolve, reject) {
    get({
      id: id,
      token: options.token,
      url: options.url,
      api: 'collections/' + options.collectionid
    }).then(resolve).catch(reject);
  });
}

function _listCollection(options, id) {
  return new Promise(function (resolve, reject) {
    get({
      id: id,
      token: options.token,
      url: options.url,
      api: 'collections'
    }).then(resolve).catch(reject);
  });
}

var opt$2 = Object.create(options);

function remove(conn) {
  var d = rest.del('https://' + conn.id + ':' + conn.token + '@' + conn.url + '/admin/' + conn.api + '.xml', opt$2);
  return new Promise(function (resolve, reject) {
    request(d).then(resolve).catch(reject);
  });
}

function _getFile(options, id) {
  return new Promise(function (resolve, reject) {
    get({
      id: id,
      token: options.token,
      url: options.url,
      api: 'files/' + options.fileid
    }).then(resolve).catch(reject);
  });
}

function _createFile(options, id) {
  return new Promise(function (resolve, reject) {
    post({
      id: id,
      token: options.token,
      url: options.url,
      api: 'files',
      obj: options.file
    }).then(resolve).catch(reject);
  });
}

function _removeFile(options, id) {
  return new Promise(function (resolve, reject) {
    remove({
      id: id,
      token: options.token,
      url: options.url,
      api: 'files/' + options.fileid
    }).then(resolve).catch(reject);
  });
}

function _uploadFile(options, id) {
  var query = {};
  query.page = options.page ? options.page : 1;
  query.per_page = options.per_page ? options.per_page : 250;

  return new Promise(function (resolve, reject) {
    get({
      id: id,
      token: options.token,
      url: options.url,
      api: 'files',
      query: query
    }).then(resolve).catch(reject);
  });
}

function _createJsTag(options, id) {
  return new Promise(function (resolve, reject) {
    post({
      id: id,
      token: options.token,
      url: options.url,
      api: 'js_tags',
      obj: options.jstag
    }).then(resolve).catch(reject);
  });
}

function _getRecurring(options, id) {
  return new Promise(function (resolve, reject) {
    get({
      id: id,
      token: options.token,
      url: options.url,
      api: 'recurring_application_charge'
    }).then(resolve).catch(reject);
  });
}

function _createRecurring(options, id) {
  var charge = {
    'recurring-application-charge': {
      'monthly': options.price
    }
  };

  return new Promise(function (resolve, reject) {
    post({
      id: id,
      token: options.token,
      url: options.url,
      api: 'recurring_application_charge',
      obj: charge
    }).then(resolve).catch(reject);
  });
}

function _removeRecurring(options, id) {
  return new Promise(function (resolve, reject) {
    remove({
      id: id,
      token: options.token,
      url: options.url,
      api: 'recurring_application_charge'
    }).then(resolve).catch(reject);
  });
}

function _getCharge(options, id) {
  return new Promise(function (resolve, reject) {
    get({
      id: id,
      token: options.token,
      url: options.url,
      api: 'application_charges/' + options.chargeid
    }).then(resolve).catch(reject);
  });
}

function _createCharge(options, id) {
  return new Promise(function (resolve, reject) {
    get({
      id: id,
      token: options.token,
      url: options.url,
      api: 'application_charges'
    }).then(resolve).catch(reject);
  });
}

function _removeCharge(options, id) {
  return new Promise(function (resolve, reject) {
    post({
      id: id,
      token: options.token,
      url: options.url,
      api: 'application_charges',
      obj: options.charge
    }).then(resolve).catch(reject);
  });
}

function _listCharge(options, id) {
  return new Promise(function (resolve, reject) {
    post({
      id: id,
      token: options.token,
      url: options.url,
      api: 'application_charges/' + options.chargeid + '/decline'
    }).then(resolve).catch(reject);
  });
}

var opt$3 = Object.create(options);

function put(conn) {
  if (conn.obj !== undefined) {
    var js2xml$$ = new js2xml.Js2Xml(Object.keys(conn.obj)[0], conn.obj[Object.keys(conn.obj)[0]]);
    opt$3.data = js2xml$$.toString();
  }
  var p = rest.put('https://' + conn.id + ':' + conn.token + '@' + conn.url + '/admin/' + conn.api + '.xml', opt$3);
  return new Promise(function (resolve, reject) {
    request(p).then(resolve).catch(reject);
  });
}

/**
 * GET /admin/discount_codes/{id}.xml
 *
 * Examples:
 *
 *      insales.getCoupon({
 *        token: '1234567890',
 *        url: 'shop-test.myinsales.ru',
 *        couponid: '1076619'
 *      }).then(response => {
 *        console.info(response.data);
 *      }).catch(err => {
 *        console.error(err);
 *      });
 * Output:
 *
 *      { 'discount-code':
 *         { 'act-once': 'false',
 *           'act-once-for-client': 'false',
 *           code: 'WWWWWW',
 *           'created-at': '2011-11-11T11:11:11+03:00',
 *           description: '',
 *           disabled: 'false',
 *           discount: '10.0',
 *           'expired-at': '2011-11-11',
 *           id: '1111111',
 *           'min-price': '0.0',
 *           'type-id': '1',
 *           'updated-at': '2011-11-11T11:11:11+03:00',
 *           worked: 'true',
 *           'discount-collections': '' } }
 * @param {Object} options
 * @param {string} options.token Required, access token for shop
 * @param {string} options.url Required, shop url
 * @param {number|string} options.id Required, coupon id
 * @param {Number|String} id Required, app id
 * @returns {Promise}
 */
function _getCoupon(options, id) {
  return new Promise(function (resolve, reject) {
    get({
      id: id,
      token: options.token,
      url: options.url,
      api: 'discount_codes/' + options.couponid
    }).then(resolve).catch(reject);
  });
}

/**
 * GET /admin/discount_codes.xml
 *
 * Examples:
 *
 *      insales.listCoupon({
 *        token: '1234567890',
 *        url: 'shop-test.myinsales.ru',
 *      }).then(response => {
 *        console.info(response.data);
 *      }).catch(err => {
 *        console.error(err);
 *      });
 * Output:
 *
 *      { 'discount-codes':
 *         { 'discount-code':
 *            [{ 'act-once': 'false',
 *              'act-once-for-client': 'false',
 *              code: 'WWWWWW',
 *              'created-at': '2011-11-11T11:11:11+03:00',
 *              description: '',
 *              disabled: 'false',
 *              discount: '10.0',
 *              'expired-at': '2011-11-11',
 *              id: '1111111',
 *              'min-price': '0.0',
 *              'type-id': '1',
 *              'updated-at': '2011-11-11T11:11:11+03:00',
 *              worked: 'true',
 *              'discount-collections': '' },
 *            { 'act-once': 'false',
 *              'act-once-for-client': 'false',
 *              code: 'WWWWWY',
 *              'created-at': '2011-11-11T11:11:11+03:00',
 *              description: '',
 *              disabled: 'false',
 *              discount: '10.0',
 *              'expired-at': '2011-11-11',
 *              id: '1111112',
 *              'min-price': '0.0',
 *              'type-id': '1',
 *              'updated-at': '2011-11-11T11:11:11+03:00',
 *              worked: 'true',
 *              'discount-collections': '' }]}}
 * @param {Object} options
 * @param {string} options.token Required, access token for shop
 * @param {string} options.url Required, shop url
 * @param {number|string} [options.page=1] Page
 * @param {number|string} [options.per_page=250] Coupons per page
 * @param {string} [options.updated_since] Coupons updated since. Format YYYY-MM-DD
 * @param {boolean} [options.deleted] Deleted coupons
 * @param {Number|String} id Required, app id
 * @returns {Promise}
 */
function _listCoupon(options, id) {
  var query = {};
  query.page = options.page ? options.page : 1;
  query.per_page = options.per_page ? options.per_page : 250;
  if (options.updated_since && options.updated_since !== false) {
    options.updated_since = options.updated_since;
  }
  if (options.deleted === true) {
    query.deleted = true;
  }

  return new Promise(function (resolve, reject) {
    get({
      id: id,
      token: options.token,
      url: options.url,
      api: 'discount_codes',
      query: query
    }).then(resolve).catch(reject);
  });
}

/**
 * POST /admin/discount_codes.xml
 *
 * Examples:
 *
 *      insales.createCoupon({
 *        token: '1234567890',
 *        url: 'shop-test.myinsales.ru',
 *        coupon: {
 *           'discount_code': {
 *             'code': 'frandom2',
 *             'act_once': 'true',
 *             'discount': 10,
 *             'type_id': 1,
 *             'description': 'API code',
 *             'disabled': 0,
 *             'expired-at': '2030-06-09'
 *           }
 *        }
 *      }).then(output => {
 *        console.info(output.response.statusCode);
 *      }).catch(err => {
 *        console.error(err);
 *      });
 * Output:
 *
 *      { 'discount-code':
 *         { 'act-once': 'true',
 *           'act-once-for-client': '',
 *           code: 'frandom2',
 *           'created-at': '2011-11-11T11:11:11+03:00',
 *           description: 'API code',
 *           disabled: 'false',
 *           discount: '10.0',
 *           'expired-at': '2030-06-09',
 *           id: '100000',
 *           'min-price': '',
 *           'type-id': '1',
 *           'updated-at': '2011-11-11T11:11:11+03:00',
 *           worked: 'true',
 *           'discount-collections': '' } }
 * @param {Object} options
 * @param {string} options.token Required, access token for shop
 * @param {string} options.url Required, shop url
 * @param {object} options.coupon Required, coupon
 * @param {Number|String} id Required, app id
 * @returns {Promise}
 */
function _createCoupon(options, id) {
  return new Promise(function (resolve, reject) {
    post({
      id: id,
      token: options.token,
      url: options.url,
      api: 'discount_codes',
      obj: options.coupon
    }).then(resolve).catch(reject);
  });
}

/**
 * DELETE /admin/discount_codes/{id}.xml
 *
 * Examples:
 *
 *      insales.removeCoupon({
 *        token: '1234567890',
 *        url: 'shop-test.myinsales.ru',
 *        couponid: '1076619'
 *      }).then(output => {
 *        console.info(output.response.statusCode);
 *      }).catch(err => {
 *        console.error(err);
 *      });
 * Output:
 *
 *      200
 * @param {Object} options
 * @param {string} options.token Required, access token for shop
 * @param {string} options.url Required, shop url
 * @param {number|string} options.couponid Required, coupon id
 * @param {Number|String} id Required, app id
 * @returns {Promise}
 */
function _removeCoupon(options, id) {
  return new Promise(function (resolve, reject) {
    remove({
      id: id,
      token: options.token,
      url: options.url,
      api: 'discount_codes/' + options.couponid
    }).then(resolve).catch(reject);
  });
}

function _editCoupon(options, id) {
  return new Promise(function (resolve, reject) {
    put({
      id: id,
      token: options.token,
      url: options.url,
      api: 'discount_codes/' + options.couponid,
      obj: options.coupon
    }).then(resolve).catch(reject);
  });
}

var InSales = function () {
  function InSales(options) {
    _classCallCheck(this, InSales);

    var _options = options || {};
    if (!_options.id) throw new Error('Missing app id');
    if (!_options.secret) throw new Error('Missing app secret');

    this.id = _options.id;
    this.secret = _options.secret;
  }

  _createClass(InSales, [{
    key: 'token',
    value: function token(tok) {
      return _token(tok, this.secret);
    }
  }, {
    key: 'getProduct',
    value: function getProduct(options) {
      return _getProduct(options, this.id);
    }
  }, {
    key: 'createProduct',
    value: function createProduct(options) {
      return _createProduct(options, this.id);
    }
  }, {
    key: 'listProduct',
    value: function listProduct(options) {
      return _listProduct(options, this.id);
    }
  }, {
    key: 'getCollection',
    value: function getCollection(options) {
      return _getCollection(options, this.id);
    }
  }, {
    key: 'listCollection',
    value: function listCollection(options) {
      return _listCollection(options, this.id);
    }
  }, {
    key: 'getFile',
    value: function getFile(options) {
      return _getFile(options, this.id);
    }
  }, {
    key: 'createFile',
    value: function createFile(options) {
      return _createFile(options, this.id);
    }
  }, {
    key: 'removeFile',
    value: function removeFile(options) {
      return _removeFile(options, this.id);
    }
  }, {
    key: 'uploadFile',
    value: function uploadFile(options) {
      return _uploadFile(options, this.id);
    }
  }, {
    key: 'getAccount',
    value: function getAccount(options) {
      return _getAccount(options, this.id);
    }
  }, {
    key: 'createJsTag',
    value: function createJsTag(options) {
      return _createJsTag(options, this.id);
    }
  }, {
    key: 'getRecurring',
    value: function getRecurring(options) {
      return _getRecurring(options, this.id);
    }
  }, {
    key: 'createRecurring',
    value: function createRecurring(options) {
      return _createRecurring(options, this.id);
    }
  }, {
    key: 'removeRecurring',
    value: function removeRecurring(options) {
      return _removeRecurring(options, this.id);
    }
  }, {
    key: 'getCharge',
    value: function getCharge(options) {
      return _getCharge(options, this.id);
    }
  }, {
    key: 'createCharge',
    value: function createCharge(options) {
      return _createCharge(options, this.id);
    }
  }, {
    key: 'removeCharge',
    value: function removeCharge(options) {
      return _removeCharge(options, this.id);
    }
  }, {
    key: 'listCharge',
    value: function listCharge(options) {
      return _listCharge(options, this.id);
    }
  }, {
    key: 'getCoupon',
    value: function getCoupon(options) {
      return _getCoupon(options, this.id);
    }
  }, {
    key: 'createCoupon',
    value: function createCoupon(options) {
      return _createCoupon(options, this.id);
    }
  }, {
    key: 'removeCoupon',
    value: function removeCoupon(options) {
      return _removeCoupon(options, this.id);
    }
  }, {
    key: 'listCoupon',
    value: function listCoupon(options) {
      return _listCoupon(options, this.id);
    }
  }, {
    key: 'editCoupon',
    value: function editCoupon(options) {
      return _editCoupon(options, this.id);
    }
  }]);

  return InSales;
}();

var index = function index(options) {
  return new InSales(options);
};

module.exports = index;
